# 1.weex子服务可调用能力

云城weex子服务的开发中，开发者可以通过 `weex.requireModule('moduleName')` 引入云城提供的插件，方便快捷的调起云城能力，如监听安卓机物理返回键点击，调用云城扫描二维码等。

## 1.1.监听安卓物理返回键点击事件

``` javascript
// vue
<script>
const globalEvent = weex.requireModule('globalEvent')

export default {
  created() {
    globalEvent.addEventListener("clickEvent", (e) => {
      // do something
    })
  }
}
</script>
```

## 1.2.扫描或生成二维码

``` javascript
// vue
<script>
const wxQRCode = weex.requireModule('wxQRCode')

export default {
  methods: {
    // 扫描二维码
    scanQRCode() {
      wxQRCode.scanQRCode(res => {
        // 返回结果： { res： "1", result: "二维码内容" }
      })
    },
    // 生成二维码
    createQRCode() {
      const param={"width":"100","height":"100","content": "二维码内容"}
      wxQRCode.createQRCode(param, res => {
        // 返回结果： { res： "1", path: "二维码路径" }
      })
    }
  }
}
</script>
```

## 1.3.上传图片

``` javascript
// vue
<script>
  const uploadFile = weex.requireModule("fileUpload")

  export default {
    methods: {
      uploadFile() {
        const params = {
          "URL": "后台服务地址",
          "file": "本地图片地址"
        }
        uploadFile.uploadFile(params, ret => {
          // 返回结果： { res: '1', returnJson: '后台返回信息' }
        })
      }
    }
  }

</script>
```

## 1.4.从相册选择图片

``` javascript
// vue
<script>
  const wxImage = weex.requireModule("image")

  export default {
    methods: {
      pickImage() {
        const type = 1 // 1:图片 2:视频 3:图片和视频
        const count = 9 // 选择的数量限制，最大为9
        wxImage.openImageVideoPicker(type, count, obj => {
          // 返回结果 { res: '1', photos: ['图片路径数组'] }
        })
      }
    }
  }
</script>
```

## 1.5.拍照

``` javascript
// vue
<script>
  const wxCamera = weex.requireModule("camera")

  export default {
    methods: {
      takePhoto() {
        wxCamera.takePicture(obj => {
          // 返回结果 {'res' : '1'， 'photos' : ['图片路径数组']}
        })
      }
    }
  }
</script>
```
<!-- takeVideo() {
  // duration为拍摄最长时间，目前支持的最大值为30，最小值为5
  // res,0为用户取消，1为用户拍摄图片，2为用户拍摄小视频
  // imagePath,当res为1，代表图片地址;当res为2,代表视频缩略图
  // videoPath,视频地址
  const duration = 30
  wxCamera.takeCamera(duration, obj => {
    // 返回结果 {'res' : '2'， 'imagePath' : 缩略图地址, 'videoPath' : 视频地址}
  })
} -->

## 1.6.预览图片

``` javascript
// vue
<script>
  const image = weex.requireModule("image")

  export default {
    methods: {
      preview() {
        // 从数组的第一张开始预览
        const images =['图片路径']
        image.preview(images)
      },
      previewForIndex() {
        // 按指定的数组下标开始预览
        const images =['图片路径']
        image.previewForIndex(images, index)
      }
    }
  }
</script>
```

## 1.7.调用云城支付

云城app为weex开发者集成了调用云城支付的能力

参数名称|类型|是否必须|描述
:--|:--|:--|:--
merchantNo|string|true|云城app分配的商户号，需与云城app对接
tradeOrderNo|string|true|业务订单号，1-32位，只能为英文、数字、下划线
amount|number|true|订单金额（0.01-20000），最多2位小数
notifyUrl|string|true|支付回调的地址，支付完成后，会回调这个地址(选填)
expireTimeMinute|number|true|订单过期时间，单位分钟，不低于5分钟，不高于1440分钟
goodsDesc|string|true|商品描述，最多20个字符

```javascript
  const ycPay =  weex.requireModule('yc_pay');
  ycPay.pay(
    {
      merchantNo: 'me***********5',
      amount: '0.01',
      tradeOrderNo: 'tral' + Date.now() + Math.floor(Math.random() * 100000),
      notifyUrl: 'http://www.baidu.com',
      expireTime: '20',
      goodsDesc: '报名费用'
    },
    res => {
      // 请注意此处返回的数据为json字符串，返回结果见下方内容
      console.log(res);
    }
  );

  // 支付成功返回
{
  "code": "0（发起支付）,-1（参数验证失败）",
  "msg": "支付提示（参数验证和支付完成都有）",
  "data": {
    "payStatus": "具体的支付状态：0（成功）,-1（失败），-2（取消）",
    "payMsg": "支付完成时的提示",
    "requestParams": {
      "merchantNo": "商户号",
      "tradeOrderNo": "业务订单号",
      "amount": "订单金额",
      "goodsDesc": "商品描述",
      "notifyUrl": "支付回调的地址",
      "expireTimeMinute ": "订单过期时间，单位分钟，不低于5分钟，不高于1440分钟"
    }
  }
}
```

## 1.8.查询订单信息

通过支付业务订单号及商户号查询对应支付的信息状态。

参数名称|类型|是否必须|描述
:--|:--|:--|:--
merchantNo|string|true|云城app分配的商户号，需与云城app对接
tradeOrderNo|string|true|业务订单号，1-32位，只能为英文、数字、下划线

```javascript
const ycPay =  weex.requireModule('yc_pay');
ycPay.queryPayResult(
  {
    merchantNo: 'm********************5',
    tradeOrderNo: 'tral0*********6'
  },
  res => {
    // 请注意此处返回的数据为json字符串，返回结果见下方内容
    console.log(res);
  }
)

// 查询成功返回
{
  "msg": "查询支付结果成功", // 提示信息：查询支付结果失败（成功）
  "code": "0", // 0(发起查询),-1（参数验证失败）
  "data": {
    "res": 0, // 查询的结果：0查询成功，-1查询失败
    "requestParams": {
      "tradeOrderNo": "T*******02", // 业务订单号
      "merchantNo": "MC*****1" // 商户号
    }, 
    "payInfo": {
      "merchantNo": "M****01", // 商户号
      "orderAmount": "0.01",  // 订单金额
      "orderNo": "TM*****02", // 业务订单号
      "orderSubject": "参加万科乐泡活动", // 商品描述
      "payMethod": "ALIPAY", // 支付方式 WXPAY：微信支付 ALIPAY：支付宝支付
      "paymentAmount": "0.01", // 实际支付金额
      "paymentNo": "10********360", // 支付订单号
      "paymentTime": "20191016141230", // 订单支付时间 格式：yyyyMMddhhmmss
      "res": 0, 
      "status": 1 // 订单状态：0(待付款)、1(已付款)、2(部分退款)、3(已退款) 、4(支付失败) 、6(已撤销)
    }
  }
}
```

## 注意

weex服务中调用了云城能力后如需进行真机调试可有如下方式：
1. 确保手机和开发使用中的电脑在同一网段，使用云城APP扫一扫功能，扫描开发启用服务下的web页面中的二维码。
  ![scanQRCode](/images/scanQRCode.png)

  web页面中的二维码

  ![page](/images/weexPage.png)

  通过云城打开服务页面后即可开始真机调试。

2. 上传打包后文件到[开放平台（生产环境）](https://icloudcity.com/iot-open)或[开放平台（uat环境）](http://icloudcitytest.vanke.com/iot-open/#/home)

# 2.公共能力

云城app除了对应给子服务提供设备能力以外，还具有自身的平台能力，按照平台能力的调用规则，即可使用云城对应的平台能力。

## 2.1.卡片消息跳转对应子服务

此能力关联REST接口[服务号向用户推送消息](/pages/development/server/restAPI/childServicePush.md)，按照接口描述调用，用户可以收到对应的卡片消息推送如下：

<img src="/images/childPush.png" width="640"/>

点击卡片消息可以跳转到对应的子服务入口页(参考：[服务创建&发布下的版本发布](/pages/platform/service/README.md#2版本发布))中，如按照推送消息示例中参数进行接口调用，现关注route字段中的value值，由path开始为向子服务中传递的三个参数：path， mobilePhone， name

```json
"route": {
    // iCloudCity://为固定内容，其后跟服务类型如 weex或web，其后跟子服务APPID，即上方appkey的值
    // 其后按序列化字符串拼接所需传递的参数，然后在子服务中进行对应的解析
    "value": "iCloudCity://weex/dae4245d-3/path=index.js&mobilePhone=13530986300&name=zs"
  }
```

可以看到如下图在子服务中可以通过解析url值获取到通过卡片消息传递进来的参数及服务自带的环境变量参数，由此子服务开发者可以通过获取到的不同参数来做对应的处理，如跳转进入次级页面，调用子服务中的其他功能等。

<img src="/images/params.png" width="640" />